#include <gnome.h>
#include <glade/glade.h>

#include <utillib.h>

#include "gdownman-marshal.h"

#include "download-properties.h"


GType           download_properties_get_type   ();
static void     download_properties_class_init (DownloadPropertiesClass *klass);
static void     download_properties_init       (DownloadProperties *dp);

gboolean    download_properties_response_cb     (GtkWidget *widget,
                                                 GdkEvent *event,
                                                 gpointer data);
void    download_properties_recursive_toggled   (GtkToggleButton *togglebutton,
                                                 gpointer data);
void    download_properties_recursive_count_value_changed   (GtkSpinButton *spinbutton,
                                                 gpointer data);
void    download_properties_speedlimit_toggled  (GtkToggleButton *togglebutton,
                                                 gpointer data);
void    download_properties_speedlimit_count_value_changed
                                                (GtkSpinButton *spinbutton,
                                                 gpointer data);

enum {
    LAST_SIGNAL
};

static gint download_properties_signals[LAST_SIGNAL] = { 0 };

static gpointer parent_class = NULL;


GType
download_properties_get_type ()
{
    static GType new_type = 0;

    if (!new_type)
    {
        static const GTypeInfo new_info =
        {
            sizeof (DownloadPropertiesClass),
            (GBaseInitFunc) NULL,
            (GBaseFinalizeFunc) NULL,
            (GClassInitFunc) download_properties_class_init,
            NULL,           /* class_finalize */
            NULL,           /* class_data */
            sizeof (DownloadProperties),
            0,              /* n_preallocs */
            (GInstanceInitFunc) download_properties_init
        };

        new_type = g_type_register_static (G_TYPE_OBJECT,
                                           "DownloadProperties",
                                           &new_info,
                                           0);
    }

    return new_type;
}

static void
download_properties_class_init (DownloadPropertiesClass *klass)
{
    parent_class = g_type_class_peek_parent (klass);
}

static void
download_properties_init (DownloadProperties *dp)
{
    dp->item = 0;
}

DownloadProperties *
download_properties_new (guint fd, GtkWindow *parent)
{
    DownloadProperties *dp = DOWNLOAD_PROPERTIES (g_object_new
                                    (download_properties_get_type (), NULL));
    GladeXML *xml;
    GtkSizeGroup* options_spin_group;

    /* load the dialog */
    xml = glade_xml_new ("download-properties.glade", NULL , NULL);

    if (!xml)
    {
        xml = glade_xml_new (GLADEDIR "/" "download-properties.glade", NULL ,
                             NULL);

        if (!xml)
        {
            g_warning ("The \"download properties dialog\" can't be loaded.");
            return NULL;
        }
    }

    dp->dialog = glade_xml_get_widget (xml, "download_properties_dialog");
    dp->vbox = glade_xml_get_widget (xml, "vbox");

    gtk_window_set_transient_for (GTK_WINDOW (dp->dialog), parent);

    dp->url_entry = glade_xml_get_widget (xml, "url_entry");
    dp->target_filename = glade_xml_get_widget (xml, "target_filename");

    /* Recursive download */
    dp->recursive = glade_xml_get_widget (xml, "recursive");
    dp->recursive_count = glade_xml_get_widget (xml, "recursive_count");
    dp->recursive_count_label = glade_xml_get_widget (xml,
                                                    "recursive_count_label");

    g_signal_connect (G_OBJECT (dp->recursive), "toggled",
                      G_CALLBACK (download_properties_recursive_toggled),
                      G_OBJECT (dp));

    g_signal_connect (G_OBJECT (dp->recursive_count), "value-changed",
                      G_CALLBACK (download_properties_recursive_count_value_changed),
                      G_OBJECT (dp));

    /* Speed limit */
    dp->speedlimit = glade_xml_get_widget (xml, "speedlimit");
    dp->speedlimit_count = glade_xml_get_widget (xml, "speedlimit_count");
    dp->speedlimit_count_label = glade_xml_get_widget (xml,
                                                    "speedlimit_count_label");

    g_signal_connect (G_OBJECT (dp->speedlimit), "toggled",
                      G_CALLBACK (download_properties_speedlimit_toggled),
                      G_OBJECT (dp));

    g_signal_connect (G_OBJECT (dp->speedlimit_count), "value-changed",
                      G_CALLBACK (download_properties_speedlimit_count_value_changed),
                      G_OBJECT (dp));

    /* Size group spin buttons */
    options_spin_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
    gtk_size_group_add_widget (options_spin_group, dp->recursive_count);
    gtk_size_group_add_widget (options_spin_group, dp->speedlimit_count);

    glade_xml_signal_autoconnect (xml);

    gtk_window_set_destroy_with_parent (GTK_WINDOW (dp->dialog), TRUE);

    g_signal_connect (G_OBJECT (dp->dialog), "delete-event",
                      G_CALLBACK (download_properties_response_cb),
                      G_OBJECT (dp->dialog));

    g_signal_connect_swapped (G_OBJECT (dp->dialog), "response",
                              G_CALLBACK (gtk_widget_hide),
                              dp->dialog);

    g_object_unref (G_OBJECT (xml));

    dp->fd = fd;

	return dp;
}

void
download_properties_start (DownloadProperties *dp)
{
    gtk_window_present (GTK_WINDOW (dp->dialog));
}

gboolean
download_properties_response_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
    gtk_widget_hide (widget);
    return TRUE;
}

void
download_properties_recursive_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    DownloadProperties *dp = DOWNLOAD_PROPERTIES (data);

    if (gtk_toggle_button_get_active (togglebutton))
    {
        gtk_widget_set_sensitive (dp->recursive_count, TRUE);
        gtk_widget_set_sensitive (dp->recursive_count_label, TRUE);
    }
    else
    {
        gtk_widget_set_sensitive (dp->recursive_count, FALSE);
        gtk_widget_set_sensitive (dp->recursive_count_label, FALSE);
    }
}

void
download_properties_recursive_count_value_changed (GtkSpinButton *spinbutton, gpointer data)
{
    DownloadProperties *dp = DOWNLOAD_PROPERTIES (data);

    if (gtk_spin_button_get_value_as_int (spinbutton) > 1)
        gtk_label_set_text (GTK_LABEL (dp->recursive_count_label),
                            "levels");
    else
        gtk_label_set_text (GTK_LABEL (dp->recursive_count_label), "level");
}

void
download_properties_speedlimit_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    DownloadProperties *dp = DOWNLOAD_PROPERTIES (data);
    ProjectItem *item = dp->item;
    guint speedlimit;
    GtkSpinButton *spinbutton;

    spinbutton = GTK_SPIN_BUTTON (dp->speedlimit_count);
    speedlimit = gtk_spin_button_get_value_as_int (spinbutton);
    speedlimit *= 1024;

    if (gtk_toggle_button_get_active (togglebutton))
    {
        gtk_widget_set_sensitive (dp->speedlimit_count, TRUE);
        gtk_widget_set_sensitive (dp->speedlimit_count_label, TRUE);
        dm_command_download_set_speed (dp->fd, item->project_id, item->id, speedlimit);
    }
    else
    {
        gtk_widget_set_sensitive (dp->speedlimit_count, FALSE);
        gtk_widget_set_sensitive (dp->speedlimit_count_label, FALSE);
        dm_command_download_set_speed (dp->fd, item->project_id, item->id, 0);
    }
}

void
download_properties_speedlimit_count_value_changed (GtkSpinButton *spinbutton,
    gpointer data)
{
    DownloadProperties *dp = DOWNLOAD_PROPERTIES (data);
    ProjectItem *item = dp->item;
    guint speedlimit;

    speedlimit = gtk_spin_button_get_value_as_int (spinbutton);
    speedlimit *= 1024;

    dm_command_download_set_speed (dp->fd, item->project_id, item->id, speedlimit);
}

void
download_properties_set (DownloadProperties *dp, ProjectItem *item)
{
    dp->item = item;

    if (item)
    {
        gtk_entry_set_text (GTK_ENTRY (dp->url_entry), item->url);
        gtk_widget_set_sensitive (dp->url_entry, TRUE);
        gtk_widget_set_sensitive (dp->speedlimit, TRUE);

        g_signal_handlers_block_by_func (dp->speedlimit,
                            download_properties_speedlimit_toggled,
                            dp);
        g_signal_handlers_block_by_func (dp->speedlimit_count,
                            download_properties_speedlimit_count_value_changed,
                            dp);

        switch (item->speedlimit)
        {
            case 0:
                gtk_toggle_button_set_active (
                                GTK_TOGGLE_BUTTON (dp->speedlimit), FALSE);
                gtk_spin_button_set_value (
                                GTK_SPIN_BUTTON (dp->speedlimit_count), 1);
                gtk_widget_set_sensitive (dp->speedlimit_count, FALSE);
                gtk_widget_set_sensitive (dp->speedlimit_count_label, FALSE);
                break;
            default:
                gtk_toggle_button_set_active (
                                GTK_TOGGLE_BUTTON (dp->speedlimit), TRUE);
                gtk_spin_button_set_value (
                                GTK_SPIN_BUTTON (dp->speedlimit_count),
                                item->speedlimit / 1024);
                gtk_widget_set_sensitive (dp->speedlimit_count, TRUE);
                gtk_widget_set_sensitive (dp->speedlimit_count_label, TRUE);
                break;
        }

        g_signal_handlers_unblock_by_func (dp->speedlimit,
                            download_properties_speedlimit_toggled,
                            dp);
        g_signal_handlers_unblock_by_func (dp->speedlimit_count,
                            download_properties_speedlimit_count_value_changed,
                            dp);
    }
}

void
download_properties_unset (DownloadProperties *dp, guint items)
{
    if (items == 0)
    {
        gtk_entry_set_text (GTK_ENTRY (dp->url_entry), "No download selected");
    }
    else
    {
        gtk_entry_set_text (GTK_ENTRY (dp->url_entry), "Multiple downloads selected");
    }

    gtk_widget_set_sensitive (dp->url_entry, FALSE);
    gtk_widget_set_sensitive (dp->speedlimit, FALSE);

    g_signal_handlers_block_by_func (dp->speedlimit,
                        download_properties_speedlimit_toggled,
                        dp);
    g_signal_handlers_block_by_func (dp->speedlimit_count,
                        download_properties_speedlimit_count_value_changed,
                        dp);

    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dp->speedlimit), FALSE);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (dp->speedlimit_count), 1);

    g_signal_handlers_unblock_by_func (dp->speedlimit,
                        download_properties_speedlimit_toggled,
                        dp);
    g_signal_handlers_unblock_by_func (dp->speedlimit_count,
                        download_properties_speedlimit_count_value_changed,
                        dp);

    gtk_widget_set_sensitive (dp->speedlimit_count, FALSE);
    gtk_widget_set_sensitive (dp->speedlimit_count_label, FALSE);
}

void
download_properties_sensitive_set (DownloadProperties *dp, gboolean setting)
{
    gtk_widget_set_sensitive (dp->vbox, setting);
}
