#include <gnome.h>
#include <glade/glade.h>

#include <utillib.h>

#include "gdownman-marshal.h"

#include "new-project.h"


GType           new_project_get_type           ();
static void     new_project_class_init         (NewProjectClass *klass);
static void     new_project_init               (NewProject *newproject);

void        new_project_response_cb            (GtkDialog *dialog,
                                                gint response,
                                                gpointer data);
gboolean    new_project_options_response_cb    (GtkWidget *widget,
                                                GdkEvent *event,
                                                gpointer data);
void        new_project_name_changed           (GtkEditable *editable,
                                                gpointer user_data);
void        new_project_defaultpath_toggled    (GtkToggleButton *togglebutton,
                                                gpointer data);
void        new_project_speedlimit_toggled     (GtkToggleButton *togglebutton,
                                                gpointer data);

enum {
    LAST_SIGNAL
};

static gint new_project_signals[LAST_SIGNAL] = { 0 };

static gpointer parent_class = NULL;


GType
new_project_get_type ()
{
    static GType new_project_type = 0;

    if (!new_project_type)
    {
        static const GTypeInfo new_project_info =
        {
            sizeof (NewProjectClass),
            (GBaseInitFunc) NULL,
            (GBaseFinalizeFunc) NULL,
            (GClassInitFunc) new_project_class_init,
            NULL,           /* class_finalize */
            NULL,           /* class_data */
            sizeof (NewProject),
            0,              /* n_preallocs */
            (GInstanceInitFunc) new_project_init
        };

        new_project_type = g_type_register_static (G_TYPE_OBJECT,
                                                   "NewProject",
                                                   &new_project_info,
                                                   0);
    }

    return new_project_type;
}

static void
new_project_class_init (NewProjectClass *klass)
{
    parent_class = g_type_class_peek_parent (klass);
}

static void
new_project_init (NewProject *newproject)
{
}

NewProject *
new_project_new (guint fd)
{
    NewProject *newproject = NEW_PROJECT (g_object_new (new_project_get_type (),
                                          NULL));
    GladeXML *xml;
	GtkWidget *options_button;

    /* load the new url dialog */
    xml = glade_xml_new ("new-project.glade", NULL , NULL);

    if (!xml)
    {
        xml = glade_xml_new (GLADEDIR "/" "new-project.glade", NULL , NULL);

        if (!xml)
        {
            g_warning ("The \"new project dialog\" can't be loaded.");
            return NULL;
        }
    }

    newproject->dialog = glade_xml_get_widget (xml, "new_project_dialog");
    newproject->name_entry = glade_xml_get_widget (xml, "name_entry");
    newproject->download_manually = glade_xml_get_widget (xml,
                                                          "download_manually");

    g_signal_connect (G_OBJECT (newproject->name_entry), "changed",
                      G_CALLBACK (new_project_name_changed),
                      G_OBJECT (newproject));

    gtk_toggle_button_set_active (
            GTK_TOGGLE_BUTTON (newproject->download_manually),
            TRUE);
    options_button = glade_xml_get_widget (xml, "options_button");

    newproject->dialog_options = glade_xml_get_widget (xml,
                                                "new_project_options_dialog");

    /* Path */
    newproject->default_path = glade_xml_get_widget (xml, "default_path");
    newproject->path_entry = glade_xml_get_widget (xml, "path_entry");

    g_signal_connect (G_OBJECT (newproject->default_path), "toggled",
                      G_CALLBACK (new_project_defaultpath_toggled),
                      G_OBJECT (newproject));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (newproject->default_path),
                                  TRUE);

    /* Speed limit */
    newproject->speedlimit = glade_xml_get_widget (xml, "speedlimit");
    newproject->speedlimit_count = glade_xml_get_widget (xml,
                                                         "speedlimit_count");
    newproject->speedlimit_count_label = glade_xml_get_widget (xml,
                                                    "speedlimit_count_label");

    g_signal_connect (G_OBJECT (newproject->speedlimit), "toggled",
                      G_CALLBACK (new_project_speedlimit_toggled),
                      G_OBJECT (newproject));

    glade_xml_signal_autoconnect (xml);

    g_signal_connect_swapped (G_OBJECT (options_button), "clicked",
                              G_CALLBACK (gtk_widget_show),
                              G_OBJECT (newproject->dialog_options));

    gtk_window_set_transient_for (GTK_WINDOW (newproject->dialog_options),
                                  GTK_WINDOW (newproject->dialog));

    gtk_window_set_destroy_with_parent (GTK_WINDOW (newproject->dialog_options),
                                        TRUE);

    g_signal_connect (G_OBJECT (newproject->dialog_options), "delete-event",
                      G_CALLBACK (new_project_options_response_cb),
                      G_OBJECT (newproject->dialog_options));

    g_signal_connect (G_OBJECT (newproject->dialog), "response",
                      G_CALLBACK (new_project_response_cb),
                      G_OBJECT (newproject));

    g_object_unref (G_OBJECT (xml));

    newproject->fd = fd;

	return newproject;
}

void
new_project_start (NewProject *newproject)
{
    gtk_widget_show (newproject->dialog);
}

void
new_project_response_cb (GtkDialog *dialog, gint response, gpointer data)
{
    NewProject *newproject = NEW_PROJECT (data);
    GString *item;
	const gchar *name, *path;

    switch (response)
    {
        case GTK_RESPONSE_OK:
            name = gtk_entry_get_text (GTK_ENTRY (newproject->name_entry));
            path = gnome_file_entry_get_full_path (
                                    GNOME_FILE_ENTRY (newproject->path_entry),
                                    FALSE);

            if ((strlen (name) > 0) && (strlen (path) > 0))
            {
                item = dm_command_project_create_new (name, path);
                dm_command_project_create_auto (item,
                            gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                                    (newproject->download_manually)));
                if (gtk_toggle_button_get_active (
                            GTK_TOGGLE_BUTTON (newproject->speedlimit)))
                {
                    gfloat speed = gtk_spin_button_get_value (
                                GTK_SPIN_BUTTON (newproject->speedlimit_count));
                    speed *= 1024;
                    dm_command_project_create_set_speed (item, (gint) speed);
                }
                dm_command_project_create (item, newproject->fd);
            }
            break;

        default:
	    break;
    }

    gtk_widget_destroy (GTK_WIDGET (dialog));
}

gboolean
new_project_options_response_cb (GtkWidget *widget, GdkEvent *event,
    gpointer data)
{
    gtk_widget_hide (widget);
    return TRUE;
}

void
new_project_name_changed (GtkEditable *editable, gpointer data)
{
    NewProject *newproject = NEW_PROJECT (data);
    GtkToggleButton *togglebutton;
    GString *buffer;
    const gchar *name;

    togglebutton = GTK_TOGGLE_BUTTON (newproject->default_path);
    if (gtk_toggle_button_get_active (togglebutton))
    {
        name = gtk_entry_get_text (GTK_ENTRY (newproject->name_entry));
        if (!name)
            return;

        buffer = g_string_new (NULL);
        g_string_printf (buffer, "%s/transfers/%s", g_get_home_dir (), name);
        gnome_file_entry_set_filename (
                                   GNOME_FILE_ENTRY (newproject->path_entry),
                                   buffer->str);
        g_string_free (buffer, TRUE);
    }
}

void
new_project_defaultpath_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    NewProject *newproject = NEW_PROJECT (data);
    GString *buffer = g_string_new (NULL);
    const gchar *name;

    if (gtk_toggle_button_get_active (togglebutton))
    {
        gtk_widget_set_sensitive (newproject->path_entry, FALSE);
    }
    else
    {
        gtk_widget_set_sensitive (newproject->path_entry, TRUE);
    }

    name = gtk_entry_get_text (GTK_ENTRY (newproject->name_entry));
    g_string_printf (buffer, "%s/transfers/%s", g_get_home_dir (), name);
    gnome_file_entry_set_filename (GNOME_FILE_ENTRY (newproject->path_entry),
                                   buffer->str);
    g_string_free (buffer, TRUE);
}

void
new_project_speedlimit_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    NewProject *newproject = NEW_PROJECT (data);

    if (gtk_toggle_button_get_active (togglebutton))
    {
        gtk_widget_set_sensitive (newproject->speedlimit_count, TRUE);
        gtk_widget_set_sensitive (newproject->speedlimit_count_label, TRUE);
    }
    else
    {
        gtk_widget_set_sensitive (newproject->speedlimit_count, FALSE);
        gtk_widget_set_sensitive (newproject->speedlimit_count_label, FALSE);
    }
}
