#include <gnome.h>
#include <glade/glade.h>

#include <utillib.h>

#include "gdownman-marshal.h"

#include "project-settings.h"
#include "new-url.h"


GType           new_url_get_type           ();
static void     new_url_class_init         (NewUrlClass *klass);
static void     new_url_init               (NewUrl *newurl);

void    new_url_response_cb                (GtkDialog *dialog, gint response,
                                            gpointer data);
gboolean    new_url_options_response_cb    (GtkWidget *widget, GdkEvent *event,
                                            gpointer data);
void    new_url_defaultfolder_toggled      (GtkToggleButton *togglebutton,
                                            gpointer data);
void    new_url_recursive_toggled          (GtkToggleButton *togglebutton,
                                            gpointer data);
void    new_url_recursive_count_value_changed   (GtkSpinButton *spinbutton,
                                                 gpointer data);
void    new_url_speedlimit_toggled         (GtkToggleButton *togglebutton,
                                            gpointer data);

enum {
    LAST_SIGNAL
};

static gint new_url_signals[LAST_SIGNAL] = { 0 };

static gpointer parent_class = NULL;


GType
new_url_get_type ()
{
    static GType new_url_type = 0;

    if (!new_url_type)
    {
        static const GTypeInfo new_url_info =
        {
            sizeof (NewUrlClass),
            (GBaseInitFunc) NULL,
            (GBaseFinalizeFunc) NULL,
            (GClassInitFunc) new_url_class_init,
            NULL,           /* class_finalize */
            NULL,           /* class_data */
            sizeof (NewUrl),
            0,              /* n_preallocs */
            (GInstanceInitFunc) new_url_init
        };

        new_url_type = g_type_register_static (G_TYPE_OBJECT,
                                               "NewUrl",
                                               &new_url_info,
                                               0);
    }

    return new_url_type;
}

static void
new_url_class_init (NewUrlClass *klass)
{
    parent_class = g_type_class_peek_parent (klass);
}

static void
new_url_init (NewUrl *newurl)
{
}

NewUrl *
new_url_new (guint project, ProjectSettings *project_settings, guint fd)
{
    NewUrl *newurl = NEW_URL (g_object_new (new_url_get_type (), NULL));
    GladeXML *xml;
	GtkWidget *options_button;
    GtkSizeGroup* options_spin_group;

    newurl->settings = project_settings;

    /* load the new url dialog */
    xml = glade_xml_new ("new-url.glade", NULL , NULL);

    if (!xml)
    {
        xml = glade_xml_new (GLADEDIR "/" "new-url.glade", NULL , NULL);

        if (!xml)
        {
            g_warning ("The \"new url dialog\" can't be loaded.");
            return NULL;
        }
    }

    newurl->dialog = glade_xml_get_widget (xml, "new_url_dialog");
    newurl->url_entry = glade_xml_get_widget (xml, "url_entry");
    newurl->download_manually = glade_xml_get_widget (xml, "download_manually");
    options_button = glade_xml_get_widget (xml, "options_button");

    newurl->dialog_options = glade_xml_get_widget (xml,
                                                   "new_url_options_dialog");

    /* Save in */
    newurl->default_folder = glade_xml_get_widget (xml, "default_folder");
    newurl->target_folder = glade_xml_get_widget (xml, "target_folder");

    g_signal_connect (G_OBJECT (newurl->default_folder), "toggled",
                      G_CALLBACK (new_url_defaultfolder_toggled),
                      G_OBJECT (newurl));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (newurl->default_folder),
                                  TRUE);

    /* Recursive download */
    newurl->recursive = glade_xml_get_widget (xml, "recursive");
    newurl->recursive_count = glade_xml_get_widget (xml, "recursive_count");
    newurl->recursive_count_label = glade_xml_get_widget (xml,
                                                    "recursive_count_label");

    g_signal_connect (G_OBJECT (newurl->recursive), "toggled",
                      G_CALLBACK (new_url_recursive_toggled),
                      G_OBJECT (newurl));

    g_signal_connect (G_OBJECT (newurl->recursive_count), "value-changed",
                      G_CALLBACK (new_url_recursive_count_value_changed),
                      G_OBJECT (newurl));

    /* Speed limit */
    newurl->speedlimit = glade_xml_get_widget (xml, "speedlimit");
    newurl->speedlimit_count = glade_xml_get_widget (xml, "speedlimit_count");
    newurl->speedlimit_count_label = glade_xml_get_widget (xml,
                                                    "speedlimit_count_label");

    g_signal_connect (G_OBJECT (newurl->speedlimit), "toggled",
                      G_CALLBACK (new_url_speedlimit_toggled),
                      G_OBJECT (newurl));

    /* Size group spin buttons */
    options_spin_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
    gtk_size_group_add_widget (options_spin_group, newurl->recursive_count);
    gtk_size_group_add_widget (options_spin_group, newurl->speedlimit_count);

    glade_xml_signal_autoconnect (xml);

    g_signal_connect_swapped (G_OBJECT (options_button), "clicked",
                              G_CALLBACK (gtk_widget_show),
                              G_OBJECT (newurl->dialog_options));

    gtk_window_set_transient_for (GTK_WINDOW (newurl->dialog_options),
                                  GTK_WINDOW (newurl->dialog));

    gtk_window_set_destroy_with_parent (GTK_WINDOW (newurl->dialog_options),
                                        TRUE);

    g_signal_connect (G_OBJECT (newurl->dialog_options), "delete-event",
                      G_CALLBACK (new_url_options_response_cb),
                      G_OBJECT (newurl->dialog_options));

    g_signal_connect (G_OBJECT (newurl->dialog), "response",
                      G_CALLBACK (new_url_response_cb),
                      G_OBJECT (newurl));

    g_object_unref (G_OBJECT (xml));

    newurl->project = project;
    newurl->fd = fd;

	return newurl;
}

void
new_url_start (NewUrl *newurl)
{
    gtk_widget_show (newurl->dialog);
}

void
new_url_response_cb (GtkDialog *dialog, gint response, gpointer data)
{
    NewUrl *newurl = NEW_URL (data);
    GString *item;
	const gchar *url;
	const gchar *path;

    switch (response)
    {
        case GTK_RESPONSE_OK:
            url = gtk_entry_get_text (GTK_ENTRY (newurl->url_entry));
            if (strlen (url) > 0)
            {
                item = dm_command_get_new (newurl->project, url);
                dm_command_get_set_stop (item,
                            gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                                    (newurl->download_manually)));
                if (!gtk_toggle_button_get_active (
                            GTK_TOGGLE_BUTTON (newurl->default_folder)))
                {
                    path = gnome_file_entry_get_full_path (
                                    GNOME_FILE_ENTRY (newurl->target_folder),
                                    FALSE);
                    if (path)
                        dm_command_get_set_localpath (item, path);
                }
                if (gtk_toggle_button_get_active (
                            GTK_TOGGLE_BUTTON (newurl->recursive)))
                {
                    dm_command_get_set_recursive (item,
                            gtk_spin_button_get_value_as_int (
                                GTK_SPIN_BUTTON (newurl->recursive_count)));
                }
                if (gtk_toggle_button_get_active (
                            GTK_TOGGLE_BUTTON (newurl->speedlimit)))
                {
                    gfloat speed = gtk_spin_button_get_value (
                                    GTK_SPIN_BUTTON (newurl->speedlimit_count));
                    speed *= 1024;
                    dm_command_get_set_speed (item, (gint) speed);
                }
                dm_command_get (item, newurl->fd);
            }
            break;

        default:
	    break;
    }

    gtk_widget_destroy (GTK_WIDGET (dialog));
}

gboolean
new_url_options_response_cb (GtkWidget *widget, GdkEvent *event, gpointer data)
{
    gtk_widget_hide (widget);
    return TRUE;
}

void
new_url_defaultfolder_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    NewUrl *newurl = NEW_URL (data);
    GString *buffer = g_string_new (NULL);
    const gchar *name;

    if (gtk_toggle_button_get_active (togglebutton))
    {
        gtk_widget_set_sensitive (newurl->target_folder, FALSE);
    }
    else
    {
        gtk_widget_set_sensitive (newurl->target_folder, TRUE);
    }

//    g_string_printf (buffer, "%s/downloads/%s", g_get_home_dir (), name);
    gnome_file_entry_set_filename (GNOME_FILE_ENTRY (newurl->target_folder),
                                project_settings_path_get (newurl->settings));
    g_string_free (buffer, TRUE);
}

void
new_url_recursive_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    NewUrl *newurl = NEW_URL (data);

    if (gtk_toggle_button_get_active (togglebutton))
    {
        gtk_widget_set_sensitive (newurl->recursive_count, TRUE);
        gtk_widget_set_sensitive (newurl->recursive_count_label, TRUE);
    }
    else
    {
        gtk_widget_set_sensitive (newurl->recursive_count, FALSE);
        gtk_widget_set_sensitive (newurl->recursive_count_label, FALSE);
    }
}

void
new_url_recursive_count_value_changed (GtkSpinButton *spinbutton, gpointer data)
{
    NewUrl *newurl = NEW_URL (data);

    if (gtk_spin_button_get_value_as_int (spinbutton) > 1)
        gtk_label_set_text (GTK_LABEL (newurl->recursive_count_label),
                            "levels");
    else
        gtk_label_set_text (GTK_LABEL (newurl->recursive_count_label), "level");
}

void
new_url_speedlimit_toggled (GtkToggleButton *togglebutton, gpointer data)
{
    NewUrl *newurl = NEW_URL (data);

    if (gtk_toggle_button_get_active (togglebutton))
    {
        gtk_widget_set_sensitive (newurl->speedlimit_count, TRUE);
        gtk_widget_set_sensitive (newurl->speedlimit_count_label, TRUE);
    }
    else
    {
        gtk_widget_set_sensitive (newurl->speedlimit_count, FALSE);
        gtk_widget_set_sensitive (newurl->speedlimit_count_label, FALSE);
    }
}
