#include <gnome.h>

#include <utillib.h>

#include "project.h"
#include "project-properties.h"


GType           project_properties_get_type           ();
static void     project_properties_class_init         (ProjectPropertiesClass *klass);
static void     project_properties_init               (ProjectProperties *project_properties);
static gint     project_properties_delete_event_cb    (GtkWidget* window,
                                                       GdkEventAny* e,
                                                       gpointer data);
void    project_properties_autodownload_set     (GObject *caller,
                                                 gpointer data);
void    project_properties_autodownload_clicked (GObject *caller,
                                                 gpointer data);
void    project_properties_maxdownloads_changed (GtkSpinButton *spin_button,
                                                 gpointer data);
gboolean    project_properties_maxdownloads_send    (gpointer data);
void        project_properties_maxdownloads_set     (GObject *caller,
                                                     gpointer data);

static gpointer parent_class = NULL;


GType project_properties_get_type ()
{
    static GType project_properties_type = 0;

    if (!project_properties_type)
    {
        static const GTypeInfo project_properties_info =
        {
            sizeof (ProjectPropertiesClass),
            (GBaseInitFunc) NULL,
            (GBaseFinalizeFunc) NULL,
            (GClassInitFunc) project_properties_class_init,
            NULL,           /* class_finalize */
            NULL,           /* class_data */
            sizeof (ProjectProperties),
            0,              /* n_preallocs */
            (GInstanceInitFunc) project_properties_init
        };

        project_properties_type = g_type_register_static (GTK_TYPE_DIALOG,
                                               "ProjectProperties",
                                               &project_properties_info,
                                               0);
    }

    return project_properties_type;
}

static void project_properties_class_init (ProjectPropertiesClass *klass)
{
    parent_class = g_type_class_peek_parent (klass);

}

static void project_properties_init (ProjectProperties *project_properties)
{
    project_properties->maxdownloads_set = 0;
}

GtkWidget *project_properties_new (Project *project, gchar *title, GtkWindow *parent)
{
    ProjectProperties *project_properties = g_object_new (project_properties_get_type (), NULL);
    GtkWidget *hbox, *vbox, *label;
    GtkObject *adjustment;

    gtk_window_set_title (GTK_WINDOW(project_properties), title);

    gtk_window_set_transient_for (GTK_WINDOW(project_properties), parent);

    gtk_dialog_add_button (GTK_DIALOG(project_properties), GTK_STOCK_CLOSE,
                           GTK_RESPONSE_ACCEPT);

    g_signal_connect_swapped (G_OBJECT (project_properties), "response",
                              G_CALLBACK (gtk_widget_hide),
                              project_properties);

	project_properties->name = gtk_label_new (NULL);

	gtk_dialog_set_default_response(GTK_DIALOG(project_properties),
                                    GTK_RESPONSE_ACCEPT);

    vbox = gtk_vbox_new (FALSE, 5);
    project_properties->vbox = vbox;
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (project_properties)->vbox),
                        vbox, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);

    hbox = gtk_hbox_new (FALSE, FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), project_properties->name, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    project_properties->autodownload = gtk_check_button_new_with_label(
                                         "Start queued downloads automatically");

    hbox = gtk_hbox_new (FALSE, FALSE);
	gtk_box_pack_start (GTK_BOX (hbox), project_properties->autodownload,
                        FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    adjustment = gtk_adjustment_new (0, 0, 10, 1, 1, 1);
    project_properties->maxdownloads = gtk_spin_button_new (
                                            GTK_ADJUSTMENT(adjustment), 0, 0);
    label = gtk_label_new("simultaneous downloads");

    hbox = gtk_hbox_new (FALSE, 5);
	gtk_box_pack_start (GTK_BOX (hbox), project_properties->maxdownloads,
                        FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    gtk_widget_show_all (vbox);

	g_signal_connect (G_OBJECT(project_properties), "delete_event",
					  G_CALLBACK(project_properties_delete_event_cb),
                      NULL);

    project_properties_project_set (project_properties, project);

    g_signal_connect (G_OBJECT(project_properties->autodownload),
                      "clicked",
					  G_CALLBACK(project_properties_autodownload_clicked),
                      project_properties);

    g_signal_connect (G_OBJECT(project_properties->maxdownloads),
                      "value_changed",
					  G_CALLBACK(project_properties_maxdownloads_changed),
                      project_properties);

    return GTK_WIDGET (project_properties);
}

static gint
project_properties_delete_event_cb(GtkWidget* window, GdkEventAny* e,
                                          gpointer data)
{
    /* Do not destroy it */
    gtk_widget_hide(window);
    return TRUE;
}

void project_properties_project_set (ProjectProperties * project_properties,
                                     Project *project)
{
    GString *buffer = g_string_new(NULL);
    Project *old_project = project_properties->project;

    project_properties->project = project;

    /* stop listening to previous selected project */
    if (old_project != NULL)
    {
        g_signal_handlers_disconnect_by_func(old_project,
                project_properties_autodownload_set,
                project_properties);

        g_signal_handlers_disconnect_by_func(old_project,
                project_properties_maxdownloads_set,
                project_properties);
    }

    /* set project name */
    g_string_sprintf(buffer, "<big><b>Project:</b> %s</big>",
                     project_name(project));

    gtk_label_set_markup (GTK_LABEL (project_properties->name), buffer->str);

    /* set autodownload */
    g_signal_handlers_block_matched (G_OBJECT (project_properties->autodownload),
                                     G_SIGNAL_MATCH_DATA,
                                     0, 0, NULL, 0, project_properties);

    gtk_toggle_button_set_active (
                        GTK_TOGGLE_BUTTON (project_properties->autodownload),
                        project_settings_autodownload_get (project->settings));

    g_signal_handlers_unblock_matched (G_OBJECT (project_properties->autodownload), 
                                       G_SIGNAL_MATCH_DATA,
                                       0, 0, NULL, 0, project_properties);

    /* set maxdownloads */
    g_signal_handlers_block_matched (G_OBJECT (project_properties->maxdownloads),
                                     G_SIGNAL_MATCH_DATA,
                                     0, 0, NULL, 0, project_properties);

    gtk_spin_button_set_value (
                        GTK_SPIN_BUTTON (project_properties->maxdownloads),
                        project_settings_maxdownloads_get (project->settings));

    g_signal_handlers_unblock_matched (G_OBJECT (project_properties->maxdownloads), 
                                       G_SIGNAL_MATCH_DATA,
                                       0, 0, NULL, 0, project_properties);

    /* listen to configuration changes */
    g_signal_connect(G_OBJECT (project), "project_autodownload",
                     G_CALLBACK (project_properties_autodownload_set),
                     project_properties);

    g_signal_connect(G_OBJECT (project), "project_maxdownloads",
                     G_CALLBACK (project_properties_maxdownloads_set),
                     project_properties);
}


void project_properties_autodownload_set (GObject *caller, gpointer data)
{
    Project *project = PROJECT (caller);
    ProjectProperties *project_properties = PROJECT_PROPERTIES (data);

    g_signal_handlers_block_matched (G_OBJECT (project_properties->autodownload),
                                     G_SIGNAL_MATCH_DATA,
                                     0, 0, NULL, 0, data);
    
    gtk_toggle_button_set_active (
                        GTK_TOGGLE_BUTTON (project_properties->autodownload),
                        project_settings_autodownload_get (project->settings));

    g_signal_handlers_unblock_matched (G_OBJECT (project_properties->autodownload), 
                                       G_SIGNAL_MATCH_DATA,
                                       0, 0, NULL, 0, data);

    gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON(project_properties->autodownload), FALSE);
}

void project_properties_autodownload_clicked (GObject *caller, gpointer data)
{
    ProjectProperties *project_properties = PROJECT_PROPERTIES (data);

    g_print("toggle button was clicked\n");
    gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON(caller), TRUE);
    project_download_auto_send(NULL, project_properties->project);
}

void project_properties_maxdownloads_changed (GtkSpinButton *spin_button,
                                              gpointer data)
{
    ProjectProperties *project_properties = PROJECT_PROPERTIES (data);

    g_print("spin button changed value\n");
    project_properties->maxdownloads_set++;
    g_timeout_add (500, project_properties_maxdownloads_send, data);
}

gboolean project_properties_maxdownloads_send (gpointer data)
{
    ProjectProperties *project_properties = PROJECT_PROPERTIES (data);

    project_properties->maxdownloads_set--;
    if (project_properties->maxdownloads_set == 0)
    {
        g_print("max download value sent\n");
        project_maxdownloads_send(project_properties->project,
            gtk_spin_button_get_value_as_int(
                GTK_SPIN_BUTTON (project_properties->maxdownloads)));
        gtk_widget_set_sensitive(project_properties->maxdownloads, FALSE);
    }

    return FALSE;
}

void project_properties_maxdownloads_set (GObject *caller, gpointer data)
{
    Project *project = PROJECT (caller);
    ProjectProperties *project_properties = PROJECT_PROPERTIES (data);

    g_signal_handlers_block_matched (G_OBJECT (project_properties->maxdownloads),
                                     G_SIGNAL_MATCH_DATA,
                                     0, 0, NULL, 0, data);

    gtk_spin_button_set_value (
                        GTK_SPIN_BUTTON (project_properties->maxdownloads),
                        project_settings_maxdownloads_get (project->settings));

    g_signal_handlers_unblock_matched (G_OBJECT (project_properties->maxdownloads), 
                                       G_SIGNAL_MATCH_DATA,
                                       0, 0, NULL, 0, data);

    gtk_widget_set_sensitive (project_properties->maxdownloads, TRUE);
}

void
project_properties_sensitive_set (ProjectProperties *project_properties,
    gboolean setting)
{
    gtk_widget_set_sensitive (project_properties->vbox, setting);
}
