#ifndef __DOWNMAND_H__
#define __DOWNMAND_H__

#include <glib-object.h>

#include "servers.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum {
    DOWNMAND_COMMAND_STOP,
    DOWNMAND_COMMAND_START,
    DOWNMAND_COMMAND_DELETE,
    DOWNMAND_COMMAND_PAUSE,
    DOWNMAND_COMMAND_QUEUE
} DownmandCommand;

typedef enum {
    DOWNMAND_EVENT_NEW,
    DOWNMAND_EVENT_START,
    DOWNMAND_EVENT_STOP,
    DOWNMAND_EVENT_DELETE,
    DOWNMAND_EVENT_PAUSE,
    DOWNMAND_EVENT_QUEUE,
    DOWNMAND_EVENT_DONE
} DownmandEvent;


#define DOWNMAND(obj)           G_TYPE_CHECK_INSTANCE_CAST (obj, downmand_get_type (), Downmand)
#define DOWNMAND_CLASS(klass)   G_TYPE_CHECK_CLASS_CAST (klass, downmand_get_type (), DownmandClass)
#define IS_DOWNMAND(obj)        G_TYPE_CHECK_INSTANCE_TYPE (obj, downmand_get_type ())


typedef struct _Downmand        Downmand;
typedef struct _DownmandClass   DownmandClass;

struct _Downmand
{
    GObject object;

    /*< private >*/

    GMainLoop* main_loop;

    /* Socket to get local connections */
    int local_socket;
    GIOChannel *g_local_socket;

    /* List of connected clients */
    GHashTable *clients;
    guint clients_count;

    /* List of all projects */
    GHashTable *projects;

    Servers *servers;

    guint count;

    guint project_id;

    /* Timeouts */
    guint scheduler_timeout;
    guint clients_timeout;
    guint save_timeout;

    gint shutdown;

    /* don't g_string_new on every func */
    GString *buffer;
};

struct _DownmandClass
{
    GObjectClass parent_class;
};

GType           downmand_get_type       (void);
Downmand*       downmand_new            (void);

/* returns 0: Ok, 1: error */
int             downmand_workspace_init         (GError **error);
int             downmand_socket_local_create    (Downmand *downmand,
                                                 GError **error);

int             downmand_socket_local   (Downmand *downmand);
void            downmand_main_loop_conf (Downmand *downmand);
GMainLoop*      downmand_main_loop      (Downmand *downmand);

GHashTable*     downmand_clients    (Downmand *downmand);
GHashTable*     downmand_projects   (Downmand *downmand);

gint        downmand_shutdown_get   (Downmand *downmand);
void        downmand_shutdown_set   (Downmand *downmand,
                                     gint i);

gint        downmand_servers_read   (Downmand *downmand, GError **error);
int         downmand_projects_read  (Downmand *downmand, GError **error);

void        downmand_command        (Downmand *downmand,
                                     unsigned int project_id_,
                                     unsigned int download_id,
                                     DownmandCommand command);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __DOWNMAND_H__ */
