#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <glib.h>

#include <utillib.h>

#include "main.h"
#include "downmand.h"
#include "debug.h"

int main(int argc, char *argv[])
{
    GError *err = NULL;
    int pid;
    Downmand *downmand;

    g_type_init();
    downmand = downmand_new();

    if (downmand_workspace_init (&err))
    {
        g_warning(err->message);
        return 1;
    }

    debug_open_log();

    if (!g_thread_supported ()) g_thread_init (NULL);

    /* We need to create a default socket
       so clients can connect to the daemon */
    if (downmand_socket_local_create(downmand, &err))
    {
        g_warning(err->message);
        return 1;
    }

    g_debug("____________________________Downmand Started____________________________\n");

    pid = fork();

    if (pid < 0)
    {
        perror("Trying to fork");
        return 1;
    }
    else if (pid > 0)
    {
        // This is the parent
        return 0;
    }

    /* read servers */
    if (downmand_servers_read (downmand, &err))
    {
        g_warning (err->message);
//FIXME: clean shutdown
        return 1;
    }

    /* read projects */
    if (downmand_projects_read (downmand, &err))
    {
        g_warning (err->message);
//FIXME: clean shutdown
        return 1;
    }

    /* create and configure the main loop */
    downmand_main_loop_conf(downmand);

    /* Here we go... */
    g_main_loop_run(downmand_main_loop(downmand));

    /* We get here when g_main_loop_quit() is called */

    return 0;
}
