#ifndef __PROJECT_H__
#define __PROJECT_H__

#include <glib-object.h>

#include "servers.h"
#include "transfer.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define PROJECT(obj)            G_TYPE_CHECK_INSTANCE_CAST (obj, project_get_type (), Project)
#define PROJECT_CLASS(klass)    G_TYPE_CHECK_CLASS_CAST (klass, project_get_type (), ProjectClass)
#define IS_PROJECT(obj)         G_TYPE_CHECK_INSTANCE_TYPE (obj, project_get_type ())


typedef struct _Project        Project;
typedef struct _ProjectClass   ProjectClass;

struct _Project
{
    GObject object;

    /*< private >*/

    GHashTable *hash;
    GHashTable *hash_active;

    GList *batch;
    GList *batch_active;

    /* Files of the project are saved in path */
    gchar *path;
    gchar *name;
    guint project_id;

    guint transfer_id;

    /* Counters */
    guint queued_transfers;
    guint active_transfers;
    GnomeVFSFileSize bytes_done;
    GnomeVFSFileSize bytes_total;

    /* Settings */
    guint max_transfers;
    guint auto_transfer;

    /* Downloaded files go to default_path */
    gchar *default_path;
    guint speedlimit;

    /* helper objects */
    Servers *servers;
};

struct _ProjectClass
{
    GObjectClass parent_class;

    /* emitted when a project changes the autotransfer setting */
    void (* project_autotransfer_changed)   (void);

    /* emitted when a project changes the maxtransfers setting */
    void (* project_maxtransfers_changed)   (void);

    /* emitted when a new transfer is added to the project */
    void (* transfer_new)       (Transfer *transfer);

    /* emitted when a transfer is stopped */
    void (* transfer_stop)      (Transfer *transfer);

    /* emitted when a transfer is finished */
    void (* transfer_done)      (Transfer *transfer);

    /* emitted when a transfer is removed from the project */
    void (* transfer_delete)    (Transfer *transfer);

    /* emitted when a transfer changes some of its propertiest */
    void (* transfer_update)    (Transfer *transfer, gint setting);
};

GType           project_get_type        (void);
Project*        project_new             (guint project_id, GError **error);

void            project_name_set        (Project *project, const gchar *name);
const gchar*    project_name_get        (Project *project);
void            project_path_set        (Project *project, const gchar *path);
const gchar*    project_path_get        (Project *project);
void            project_default_path_set    (Project *project,
                                             const gchar *path);
const gchar*    project_default_path_get    (Project *project);
void            project_speedlimit_set  (Project *project, guint speedlimit);

GHashTable* project_hash            (Project *project);
GList*      project_batch           (Project *project);
GList*      project_batch_active    (Project *project);

guint       project_queued_transfers    (Project *project);

guint       project_size            (Project *project);

void    project_scheduler       (Project *project);
gint    project_command_transfer_new    (Project *project, gchar *command,
                                         GError **error);
gint    project_command_transfer_start  (Project *project, guint id,
                                         GError **error);
gint    project_command_transfer_stop   (Project *project, guint id,
                                         GError **error);
gint    project_command_transfer_queue  (Project *project, guint id,
                                         GError **error);
gint    project_command_transfer_delete (Project *project, guint id,
                                         GError **error);
void    project_command_transfer_speed  (Project *project, guint transfer_id,
                                         guint setting,
                                         GError **err);

gint    project_preferences_save    (Project *project);
gint    project_session_save        (Project *project);

gint    project_preferences_load    (Project *project);
gint    project_session_load        (Project *project);

guint           project_id          (Project *project);
const gchar*    project_name        (Project *project);
gint            project_auto        (Project *project);
void            project_auto_set    (Project *project, guint setting,
                                     GError **error);
guint           project_maxtransfers_get    (Project *project);
void            project_maxtransfers_set    (Project *project,
                                             guint maxtransfers,
                                             GError **error);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __PROJECT_H__ */
