#include <stdio.h>
#include <glib.h>

#include <utillib.h>

#include "downmand-marshal.h"

#include "servers.h"


GType           servers_get_type            ();
static void     servers_class_init          (ServersClass *klass);
static void     servers_init                (Servers *servers);

enum {
    SERVERS_NEW,
    SERVERS_CHANGED,
    LAST_SIGNAL
};

static gint servers_signals[LAST_SIGNAL] = { 0 };

static gpointer parent_class = NULL;

static Servers *servers_static = NULL;


GType
servers_get_type ()
{
    static GType type = 0;

    if (!type)
    {
        static const GTypeInfo info =
        {
            sizeof (ServersClass),
            (GBaseInitFunc) NULL,
            (GBaseFinalizeFunc) NULL,
            (GClassInitFunc) servers_class_init,
            NULL,           /* class_finalize */
            NULL,           /* class_data */
            sizeof (Servers),
            0,              /* n_preallocs */
            (GInstanceInitFunc) servers_init
        };

        type = g_type_register_static (G_TYPE_OBJECT,
                                       "Servers",
                                       &info,
                                       0);
    }

    return type;
}

static void
servers_class_init (ServersClass *klass)
{
    GObjectClass *object_class;

    object_class = (GObjectClass*) klass;

    parent_class = g_type_class_peek_parent (klass);

    servers_signals[SERVERS_NEW] = g_signal_new (
            "servers_new",
            G_OBJECT_CLASS_TYPE (klass),
            G_SIGNAL_RUN_FIRST,
            G_STRUCT_OFFSET (ServersClass, servers_new),
            NULL, NULL,
            downmand_marshal_VOID__POINTER,
            G_TYPE_NONE,
            1, G_TYPE_POINTER);

    servers_signals[SERVERS_CHANGED] = g_signal_new (
            "servers_changed",
            G_OBJECT_CLASS_TYPE (klass),
            G_SIGNAL_RUN_FIRST,
            G_STRUCT_OFFSET (ServersClass, servers_changed),
            NULL, NULL,
            downmand_marshal_VOID__POINTER_INT,
            G_TYPE_NONE,
            2, G_TYPE_POINTER, G_TYPE_INT);

    klass->servers_new = NULL;
    klass->servers_changed = NULL;
}

static void
servers_init (Servers *servers)
{
    servers->hash = g_hash_table_new (g_str_hash, g_str_equal);
}

Servers *
servers_new (void)
{
    if (servers_static == NULL)
    {
        servers_static = SERVERS (g_object_new (servers_get_type (), NULL));
    }

    return servers_static;
}

/* server_url is a string of the form http://www.gnome.org
 */
gboolean
servers_connection_limit_reached (Servers *servers, const gchar *server_url)
{
    GHashTable *hash = servers->hash;
    Server *server;

    server = g_hash_table_lookup (hash, server_url);

    if (!server)
        return FALSE;

    if (server->max_connections == -1)
        return FALSE;

    if (server->max_connections == 0)
        return TRUE;

    if (server->current_connections >= server->max_connections)
        return TRUE;

    return FALSE;
}

void
servers_connection_start (Servers *servers, const gchar *server_url)
{
    GHashTable *hash = servers->hash;
    Server *server;

    server = g_hash_table_lookup (hash, server_url);

    if (!server)
    {
        server = g_new (Server, 1);
        /* By setting max_connections to -1 we mean that the value is unset.
         * This means that we won't save this server in the servers file.
         */
        server->server_url = g_strdup (server_url);
        server->max_connections = -1;
        server->current_connections = 1;

        g_hash_table_insert (hash, server->server_url, server);
        g_signal_emit (G_OBJECT (servers), servers_signals [SERVERS_NEW], 0,
                       server);
    }
    else
    {
        server->current_connections ++;
        g_signal_emit (G_OBJECT (servers), servers_signals [SERVERS_CHANGED], 0,
                       server,
                       DM_COMMAND_SERVER_CURRENT_CONNECTIONS);
    }
}

void
servers_connection_end (Servers *servers, const gchar *server_url)
{
    GHashTable *hash = servers->hash;
    Server *server;

    server = g_hash_table_lookup (hash, server_url);

    if (!server)
    {
        /* Wops! programming error */
        g_printerr ("Tried to end a connection that is not accounted.");
        return;
    }

    server->current_connections --;
    g_signal_emit (G_OBJECT (servers), servers_signals [SERVERS_CHANGED], 0,
                   server,
                   DM_COMMAND_SERVER_CURRENT_CONNECTIONS);
}

void
servers_server_new (Servers *servers, Server *server)
{
    g_hash_table_insert (servers->hash, server->server_url, server);
}

void
servers_max_connections (Servers *servers, gchar *command)
{
    GHashTable *hash = servers->hash;
    Server *server;
    gchar *space;
    gint max_conn;

    space = g_strrstr (command, " ");
    if (!space)
        return;
    space[0] = '\0';
    sscanf (space + 1, "%i", &max_conn);

    server = g_hash_table_lookup (hash, command);

    if (!server)
    {
        server = g_new (Server, 1);
        server->server_url = g_strdup (command);
        server->max_connections = max_conn;
        server->current_connections = 0;

        g_hash_table_insert (hash, server->server_url, server);
        g_signal_emit (G_OBJECT (servers), servers_signals [SERVERS_NEW], 0,
                       server);
    }
    else
    {
        server->max_connections = max_conn;

        g_signal_emit (G_OBJECT (servers), servers_signals [SERVERS_CHANGED], 0,
                       server,
                       DM_COMMAND_SERVER_MAX_CONNECTIONS);
    }
}
