#ifndef __TRANSFER_THREAD_H__
#define __TRANSFER_THREAD_H__

#include <glib-object.h>
#include <libgnomevfs/gnome-vfs-uri.h>
#include <libgnomevfs/gnome-vfs-file-info.h>
#include <libgnomevfs/gnome-vfs-async-ops.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define TRANSFER_THREAD(obj)            G_TYPE_CHECK_INSTANCE_CAST (obj,    \
                                            transfer_thread_get_type (),    \
                                            TransferThread)
#define TRANSFER_THREAD_CLASS(klass)    G_TYPE_CHECK_CLASS_CAST (klass,     \
                                            transfer_thread_get_type (),    \
                                            TransferThreadClass)
#define IS_TRANSFER_THREAD(obj)         G_TYPE_CHECK_INSTANCE_TYPE (obj,    \
                                            transfer_thread_get_type ())


typedef struct _TransferThread       TransferThread;
typedef struct _TransferThreadClass  TransferThreadClass;

struct _TransferThread
{
    GObject object;

    /*< private >*/
    GnomeVFSURI *source_uri;
    GnomeVFSURI *target_uri;

    GnomeVFSAsyncHandle *source_handle;
    GnomeVFSAsyncHandle *target_handle;

	GnomeVFSFileSize bytes_start;
	GnomeVFSFileSize bytes_stop;
	GnomeVFSFileSize bytes_done;

    /* the limit set on the transfer */
    guint speedlimit;
    /* the chunk we can still transfer */
    gint speedlimit_left;
    /* last time we transfered some chunk */
    struct timeval oldtime;

	char *buffer;

    enum dm_transfer_status status;

    gboolean stop;
    gboolean create_target;
};

struct _TransferThreadClass
{
    GObjectClass parent_class;

    /* Emitted when the target file is created */
    void (* target_created_signal)     (void);
    /* Emitted when the thread starts transfering */
    void (* transfering_signal)        (void);
    /* Emitted when the thread stops */
    void (* stop_signal)               (void);
    /* Emitted when the thread ends the transfer */
    void (* done_signal)               (void);
    /* Emitted when the thread ends because of an error */
    void (* error_signal)               (void);
};

GType           transfer_thread_get_type       (void);
TransferThread* transfer_thread_new            ();

void    transfer_thread_start           (TransferThread *transfer_thread);

void    transfer_thread_target_uri_set  (TransferThread *transfer_thread,
                                         GnomeVFSURI *target_uri);
void    transfer_thread_source_uri_set  (TransferThread *transfer_thread,
                                         GnomeVFSURI *source_uri);

void    transfer_thread_bytes_start_set (TransferThread *transfer_thread,
                                         GnomeVFSFileSize bytes_start);
void    transfer_thread_bytes_stop_set  (TransferThread *transfer_thread,
                                         GnomeVFSFileSize bytes_stop);

GnomeVFSFileSize    transfer_thread_bytes_done_get
                                        (TransferThread *transfer_thread);
void    transfer_thread_speedlimit_set  (TransferThread *transfer_thread,
                                         guint speedlimit);

void    transfer_thread_target_create   (TransferThread *transfer_thread,
                                         gboolean create_target);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __TRANSFER_THREAD_H__ */
