#include <stdlib.h>
#include <unistd.h>

#include <glib/gstring.h>

#include "commands.h"

void    dm_send     (gint fd, GString *buffer);

GString *
dm_command_get_new (guint project, const gchar *url)
{
    GString *item = g_string_new (NULL);

    g_string_printf (item, "%u %u %u %s", DM_COMMAND_ITEM_NEW, project,
                     DM_COMMAND_ITEM_URL, url);

    return item;
}

void
dm_command_get_set_localpath (GString *item, const gchar *localpath)
{
    g_string_append_printf (item, "|%u %s", DM_COMMAND_ITEM_PATH,
                            localpath);
}

void
dm_command_get_set_stop (GString *item, gboolean stop)
{
    g_string_append_printf (item, "|%u %i", DM_COMMAND_ITEM_STOP, stop);
}

void
dm_command_get_set_recursive (GString *item, guint recursive)
{
    g_string_append_printf (item, "|%u %u", DM_COMMAND_ITEM_RECURSIVE,
                            recursive);
}

void
dm_command_get_set_speed (GString *item, guint speed)
{
    g_string_append_printf (item, "|%u %u", DM_COMMAND_ITEM_SPEED, speed);
}

gint
dm_command_get (GString *item, guint fd)
{
    g_string_append_printf (item, "\n");
    dm_send (fd, item);
    g_string_free (item, TRUE);
    return 0;
}

void
dm_command_download_set_speed (gint fd, guint project, guint download, guint speed)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %u %u\n", DM_COMMAND_ITEM_SPEED,
                     project,
                     download,
                     speed);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);
}



gint
dm_command_download_start (gint fd, guint project, guint download)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %u\n", DM_COMMAND_ITEM_START, project,
                     download);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_download_stop (gint fd, guint project, guint download)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %u\n", DM_COMMAND_ITEM_STOP, project,
                     download);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_download_queue (gint fd, guint project, guint download)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %u\n", DM_COMMAND_ITEM_QUEUE, project,
                     download);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_download_delete (gint fd, guint project, guint download)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %u\n", DM_COMMAND_ITEM_DELETE, project,
                     download);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_server_events (gint fd, guint receive)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u\n", DM_COMMAND_SERVER_EVENTS, receive);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_server_max_connections (gint fd, gchar *server, gint max_conn)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %s %i\n", DM_COMMAND_SERVER_MAX_CONNECTIONS,
                     server,
                     max_conn);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_batchstatus (gint fd, guint time, const gchar *batch)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %s\n", DM_COMMAND_CONFIG_BATCHSTATUS, time,
                     (batch) ? batch : "");
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_batchprogress (gint fd, guint time, const gchar *batch)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %s\n", DM_COMMAND_CONFIG_BATCHPROGRESS,
                     time,
                     (batch) ? batch : "");
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_events (gint fd, guint receive, const gchar *batch)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %s\n", DM_COMMAND_CONFIG_EVENTS, receive,
                     (batch) ? batch : "");
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_projects_events (gint fd, guint receive)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u\n", DM_COMMAND_CONFIG_PROJECT_EVENTS,
                     receive);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

GString *
dm_command_project_create_new (const gchar *name, const gchar *path)
{
    GString *item = g_string_new (NULL);

    g_string_printf (item, "%u %u %s|%u %s", DM_COMMAND_PROJECT_NEW,
                     DM_COMMAND_PROJECT_NAME, name,
                     DM_COMMAND_PROJECT_PATH, path);

    return item;
}

void
dm_command_project_create_auto (GString *item, guint setting)
{
    g_string_append_printf (item, "|%u %u", DM_COMMAND_PROJECT_AUTO, setting);
}

void
dm_command_project_create_set_speed (GString *item, guint speed)
{
    g_string_append_printf (item, "|%u %u", DM_COMMAND_PROJECT_SPEED, speed);
}

gint
dm_command_project_create (GString *item, guint fd)
{
    g_string_append_printf (item, "\n");
    dm_send (fd, item);
    g_string_free (item, TRUE);
    return 0;
}

gint
dm_command_project_delete (gint fd, guint project)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u\n", DM_COMMAND_PROJECT_DELETE, project);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_auto (gint fd, guint project, guint setting)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %u\n", DM_COMMAND_PROJECT_AUTO, project,
                     setting);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

gint
dm_command_maxdownloads (gint fd, guint project, guint maxdownloads)
{
    GString *buffer = g_string_new (NULL);

    g_string_printf (buffer, "%u %u %u\n", DM_COMMAND_PROJECT_MAXTRANSFERS,
                     project,
                     maxdownloads);
    dm_send (fd, buffer);
    g_string_free (buffer, TRUE);

    return 0;
}

static gchar *
dm_transfer_status_strings[] =
{
	"Done",

	"Stopping",
	"Stopped",
	"Queued",
	"Init",
	"Starting",
	"Resolving",
	"Connecting",
	"Waiting data",
	"Logging in",
	"Downloading",

	"",

	"Error opening source file",
	"Error opening target file",
	"Error creating socket",
	"Error resolving host",
	"Error connecting to host",
	"Error login",
	"Error getting head",
	"Error getting data",
    "Error reading",
    "Error writing",

	"Connection closed by host"
};

const gchar *
dm_transfer_status_string (enum dm_transfer_status status)
{
    if ((status >= 0) && (status < ERROR_LAST))
        return dm_transfer_status_strings[status];
    else
        return 0;
}

void
dm_send (gint fd, GString *buffer)
{
    gint wrote = 0, i;

    while (wrote < buffer->len)
    {
        i = write (fd, buffer->str + wrote, buffer->len - wrote);
        if (i > 0)
            wrote += i;
    }
}
